@@ -9,7 +9,7 @@
  */
 
 /*
- * $Id: main.c,v 1.117 2005/05/09 22:20:21 florian Exp $
+ * $Id: main.c,v 1.119 2005/06/03 22:37:11 florian Exp $
  *
  */
 
@@ -43,8 +43,6 @@
 #include "finddlg.h"
 
 #define MY_PIXMAPS_DIR "gpe-contacts/"
-#define TAB_CONFIG_LOCAL ".contacts-tab"
-#define TAB_CONFIG_GLOBAL "/etc/gpe/contacts-tab.conf"
 
 GtkWidget *categories_smenu;
 GtkWidget *mainw;
@@ -67,7 +65,7 @@ struct gpe_icon my_icons[] = {
   {"frame", MY_PIXMAPS_DIR "frame"},
   {"notebook", MY_PIXMAPS_DIR "notebook"},
   {"entry", MY_PIXMAPS_DIR "entry"},
-  {"icon", "gpe-contacts" },
+  {"icon", PREFIX "/share/pixmaps/gpe-contacts.png" },
   {NULL, NULL}
 };
 
@@ -662,9 +660,7 @@ selection_made (GtkTreeSelection *sel, GObject *o)
       gtk_tree_model_get (model, &iter, 1, &id, -1);
 
       p = db_get_by_uid (id);
-    
       show_details (p);
-
       discard_person (p);
 		
       gtk_widget_set_sensitive (edit_button, TRUE);
